/**
 * @file include/gbMath.hpp
 *
 * ghulbusMath - The Ghulbus Math Library
 *
 * @brief gbMath Main header file
 *
 * @todo Test and verify
 *
 * @version 0.1
 * @author Andreas Weis
 *
 */
#ifndef _GHULBUSMATH_HPP_INCLUDE_GUARD_
#define _GHULBUSMATH_HPP_INCLUDE_GUARD_

#define _USE_MATH_DEFINES
#include <cmath>
#include <iostream>

namespace GhulbusMath {
		
	template<typename T>
	class gbVector2 {
	public:
#ifdef WIN32
//disable warning: nonstandard usage of unnamed struct/union
#pragma warning(disable: 4201)
#endif
		union {
			struct {
				T _x, _y;
			};
			T _p[2];
		};
#ifdef WIN32
#pragma warning(default: 4201)
#endif
	public:
		gbVector2(T const& x, T const& y)
			:_x(x), _y(y)
		{ ; }
		gbVector2(gbVector2 const& rhs)
			:_x(rhs._x), _y(rhs._y)
		{ ; }
		template<typename T_2>
		gbVector2& operator =(gbVector2<T_2> const& rhs) {
			if( static_cast<void const*>(this) != static_cast<void const*>(&rhs) ) {
				_x = static_cast<T>( rhs._x );
				_y = static_cast<T>( rhs._y );
			}
			return *this;
		}
		T GetX() const { return _x; }
		T GetY() const { return _y; }
		T const* GetV() const { return _p; }
		void SetX(T const& x) { _x = x; }
		void SetY(T const& y) { _y = y; }
		void Set(T const& x, T const& y) { _x = x;  _y = y; }

		gbVector2 operator +(gbVector2 const& rhs) const {
			return gbVector2((_x+rhs._x), (_y+rhs._y));
		}
		gbVector2 operator -(gbVector2 const& rhs) const {
			return gbVector2((_x-rhs._x), (_y-rhs._y));
		}
		gbVector2 operator *(T const& rhs) const {
			return gbVector2((_x*rhs), (_y*rhs));
		}
		gbVector2 operator /(T const& rhs) const {
			return gbVector2((_x/rhs), (_y/rhs));
		}
		T operator *(gbVector2 const& rhs) const {
			return T((_x * rhs._x) + (_y * rhs._y));
		}
		gbVector2& operator +=(gbVector2 const& rhs) {
			_x = _x + rhs._x;
			_y = _y + rhs._y;
			return *this;
		}
		gbVector2& operator -=(gbVector2 const& rhs) {
			_x = _x - rhs._x;
			_y = _y - rhs._y;
			return *this;
		}
		gbVector2& operator *=(T const& rhs) {
			_x = _x * rhs;
			_y = _y * rhs;
			return *this;
		}
		gbVector2& operator /=(T const& rhs) {
			_x = _x / rhs;
			_y = _y / rhs;
			return *this;
		}
		bool operator ==(gbVector2 const& rhs) const {
			if(this == &rhs) { return true; }
			return( (_x == rhs._x) && (_y == rhs._y) );
		}
		bool operator !=(gbVector2 const& rhs) const {
			return(!(this->operator==(rhs)));
		}
		T GetSquareLen() const {
			return (_x*_x + _y*_y); 
		}
		T GetLen() const {
			return static_cast<T>( sqrt( static_cast<double>( _x*_x + _y*_y ) ) );
		}
		gbVector2& Normalize() {
			T len = GetLen();
			_x = _x / len;
			_y = _y / len;
			return *this;
		}
	};
	template<typename T>
	::std::ostream& operator<<(::std::ostream& os, gbVector2<T> const& v) {
		return os << "gbVector2 - X:" << v.GetX() << " Y:" << v.GetY();
	}

	template<typename T>
	class gbVector3 {
	public:
#ifdef WIN32
//disable warning: nonstandard usage of unnamed struct/union
#pragma warning(disable: 4201)
#endif
		union {
			struct {
				T _x, _y, _z;
			};
			T _p[3];
		};
#ifdef WIN32
#pragma warning(default: 4201)
#endif
	public:
		gbVector3(T const& x, T const& y, T const& z)
			:_x(x), _y(y), _z(z)
		{ ; }
		gbVector3(gbVector3 const& rhs)
			:_x(rhs._x), _y(rhs._y), _z(rhs._z)
		{ ; }
		template<typename T_2>
		gbVector3& operator =(gbVector3<T_2> const& rhs) {
			if( static_cast<void const*>(this) != static_cast<void const*>(&rhs) ) {
				_x = static_cast<T>( rhs._x );
				_y = static_cast<T>( rhs._y );
				_z = static_cast<T>( rhs._z );
			}
			return *this;
		}
		T GetX() const { return _x; }
		T GetY() const { return _y; }
		T GetZ() const { return _z; }
		T const* GetV() const { return _p; }
		void SetX(T const& x) { _x = x; }
		void SetY(T const& y) { _y = y; }
		void SetZ(T const& z) { _z = z; }
		void Set(T const& x, T const& y, T const& z) { _x = x;  _y = y;  _z = z; }

		gbVector3 operator +(gbVector3 const& rhs) const {
			return gbVector3((_x+rhs._x), (_y+rhs._y), (_z+rhs._z));
		}
		gbVector3 operator -(gbVector3 const& rhs) const {
			return gbVector3((_x-rhs._x), (_y-rhs._y), (_z-rhs._z));
		}
		gbVector3 operator *(T const& rhs) const {
			return gbVector3((_x*rhs), (_y*rhs), (_z*rhs));
		}
		gbVector3 operator /(T const& rhs) const {
			return gbVector3((_x/rhs), (_y/rhs), (_z/rhs));
		}
		T operator *(gbVector3 const& rhs) const {
			return T((_x * rhs._x) + (_y * rhs._y) + (_z * rhs._z) );
		}
		gbVector3& operator +=(gbVector3 const& rhs) {
			_x = _x + rhs._x;
			_y = _y + rhs._y;
			_z = _z + rhs._z;
			return *this;
		}
		gbVector3& operator -=(gbVector3 const& rhs) {
			_x = _x - rhs._x;
			_y = _y - rhs._y;
			_z = _z - rhs._z;
			return *this;
		}
		gbVector3& operator *=(T const& rhs) {
			_x = _x * rhs;
			_y = _y * rhs;
			_z = _z * rhs;
			return *this;
		}
		gbVector3& operator /=(T const& rhs) {
			_x = _x / rhs;
			_y = _y / rhs;
			_z = _z / rhs;
			return *this;
		}
		bool operator ==(gbVector3 const& rhs) const {
			if(this == &rhs) { return true; }
			return( (_x == rhs._x) && (_y == rhs._y) && (_z == rhs._z) );
		}
		bool operator !=(gbVector3 const& rhs) const {
			return(!(this->operator==(rhs)));
		}
		gbVector3 CrossProduct(gbVector3 const& rhs) const {
			return gbVector3( ( (_y * rhs._z) - (_z * rhs._y) ),
							  ( (_z * rhs._x) - (_x * rhs._z) ),
							  ( (_x * rhs._y) - (_y * rhs._x) ) );
		}
		gbVector3& CrossProductOnSelf(gbVector3 const& rhs) {
			T t_x = _x;
			T t_y = _y;
			T t_z = _z;
			_x = ( (t_y * rhs._z) - (t_z * rhs._y) );
			_y = ( (t_z * rhs._x) - (t_x * rhs._z) );
			_z = ( (t_x * rhs._y) - (t_y * rhs._x) );
			return *this;
		}
		T GetSquareLen() const {
			return (_x*_x + _y*_y + _z*_z); 
		}
		T GetLen() const {
			return static_cast<T>( sqrt( static_cast<double>( _x*_x + _y*_y + _z*_z ) ) );
		}
		gbVector3& Normalize() {
			T len = GetLen();
			_x = _x / len;
			_y = _y / len;
			_z = _z / len;
			return *this;
		}
		gbVector3& Homogenize() { 
			_x = _x / _z;
			_y = _y / _z;
			_z = _z / _z;
			return *this;
		}
	};
	template<typename T>
	::std::ostream& operator<<(::std::ostream& os, gbVector3<T> const& v) {
		return os << "gbVector3 - X:" << v.GetX() << " Y:" << v.GetY() << " Z:" << v.GetZ();
	}

	template<typename T>
	class gbVector4 {
	public:
#ifdef WIN32
//disable warning: nonstandard usage of unnamed struct/union
#pragma warning(disable: 4201)
#endif
		union {
			struct {
				T _x, _y, _z, _w;
			};
			T _p[4];
		};
#ifdef WIN32
#pragma warning(default: 4201)
#endif
	public:
		inline gbVector4(T const& x, T const& y, T const& z, T const& w)
			:_x(x), _y(y), _z(z), _w(w)
		{ ; }
		gbVector4(gbVector4 const& rhs)
			:_x(rhs._x), _y(rhs._y), _z(rhs._z), _w(rhs._w)
		{ ; }
		template<typename T_2>
		gbVector4 operator =(gbVector4<T_2> const& rhs) {
			if( static_cast<void const*>(this) != static_cast<void const*>(&rhs) ) {
				_x = static_cast<T>( rhs._x );
				_y = static_cast<T>( rhs._y );
				_z = static_cast<T>( rhs._z );
				_w = static_cast<T>( rhs._w );
			}
			return *this;
		}
		template<typename T_2>
		gbVector4 operator =(gbVector3<T_2> const& rhs) {
			_x = static_cast<T>( rhs.GetX() );
			_y = static_cast<T>( rhs.GetY() );
			_z = static_cast<T>( rhs.GetZ() );
			_w = static_cast<T>( 0 );
			return *this;
		}
		T GetX() const { return _x; }
		T GetY() const { return _y; }
		T GetZ() const { return _z; }
		T GetW() const { return _w; }
		T const* GetV() const { return _p; }
		void SetX(T const& x) { _x = x; }
		void SetY(T const& y) { _y = y; }
		void SetZ(T const& z) { _z = z; }
		void SetW(T const& w) { _w = w; }
		void Set(T const& x, T const& y, T const& z, T const& w) { _x=x; _y=y; _z=z; _w=w; }
		gbVector4 operator +(gbVector4 const& rhs) const {
			return gbVector4( (_x+rhs._x), (_y+rhs._y), (_z+rhs._z), (_w+rhs._w) );
		}
		gbVector4 operator -(gbVector4 const& rhs) const {
			return gbVector4( (_x-rhs._x), (_y-rhs._y), (_z-rhs._z), (_w-rhs._w) );
		}
		gbVector4 operator *(T const& rhs) const {
			return gbVector4( (_x*rhs), (_y*rhs), (_z*rhs), (_w*rhs) );
		}
		gbVector4 operator /(T const& rhs) const {
			return gbVector4( (_x/rhs), (_y/rhs), (_z/rhs), (_w/rhs) );
		}
		T operator *(gbVector4 const& rhs) const {
			return T( (_x*rhs._x) + (_y*rhs._y) + (_z*rhs._z) + (_w*rhs._w) );
		}
		gbVector4& operator +=(gbVector4 const& rhs) {
			_x = _x + rhs._x;
			_y = _y + rhs._y;
			_z = _z + rhs._z;
			_w = _w + rhs._w;
			return *this;
		}
		gbVector4& operator -=(gbVector4 const& rhs) {
			_x = _x - rhs._x;
			_y = _y - rhs._y;
			_z = _z - rhs._z;
			_w = _w - rhs._w;
			return *this;
		}
		gbVector4& operator *=(T const& rhs) {
			_x = _x * rhs;
			_y = _y * rhs;
			_z = _z * rhs;
			_w = _w * rhs;
			return *this;
		}
		gbVector4& operator /=(T const& rhs) {
			_x = _x / rhs;
			_y = _y / rhs;
			_z = _z / rhs;
			_w = _w / rhs;
			return *this;
		}
		bool operator ==(gbVector4 const& rhs) const {
			if(this == &rhs) { return true; }
			return( (_x == rhs._x) && (_y == rhs._y) && (_z == rhs._z) && (_w == rhs._w) );
		}
		bool operator !=(gbVector4 const& rhs) const {
			return(!(this->operator==(rhs)));
		}
		gbVector4 CrossProduct(gbVector4 const& rhs) const {
			return gbVector4( ( (_y * rhs._z) - (_z * rhs._y) ),
							  ( (_z * rhs._x) - (_x * rhs._z) ),
							  ( (_x * rhs._y) - (_y * rhs._x) ),
							  static_cast<T>( 0 )                );
		}
		gbVector4& CrossProductOnSelf(gbVector4 const& rhs) {
			T t_x = _x;
			T t_y = _y;
			T t_z = _z;
			_x = ( (t_y * rhs._z) - (t_z * rhs._y) );
			_y = ( (t_z * rhs._x) - (t_x * rhs._z) );
			_z = ( (t_x * rhs._y) - (t_y * rhs._x) );
			_w = static_cast<T>(0);
			return *this;
		}
		//gbVector4& operator *=(gbMatrix4 const&);		///@todo
		//gbMatrix4 operator *(gbMatrix4 const&) const;
		T GetSquareLen() const {
			return (_x*_x + _y*_y + _z*_z + _w*_w);
		}
		T GetSquareLen3() const {
			return (_x*_x + _y*_y + _z*_z);
		}
		T GetLen() const {
			return static_cast<T>( sqrt( static_cast<double>( _x*_x + _y*_y + _z*_z + _w*_w ) ) );
		}
		T GetLen3() const {
			return static_cast<T>( sqrt( static_cast<double>( _x*_x + _y*_y + _z*_z ) ) );
		}
		gbVector4& Normalize() {
			T len = GetLen();
			_x = _x / len;
			_y = _y / len;
			_z = _z / len;
			_w = _w / len;
			return *this;
		}
		gbVector4& Normalize3() {
			T len = GetLen3();
			_x = _x / len;
			_y = _y / len;
			_z = _z / len;
			return *this;
		}
		gbVector4& Homogenize() { 
			_x = _x / _w;
			_y = _y / _w;
			_z = _z / _w;
			_w = _w / _w;
			return *this;
		}
	};
	template<typename T>
	::std::ostream& operator<<(::std::ostream& os, gbVector4<T> const& v) {
		return os << "gbVector4 - X:" << v.GetX() << " Y:" << v.GetY() 
			<< " Z:" << v.GetZ() << " W:" << v.GetW();
	}
	
	template<typename T>
	class gbMatrix4
	{
	public:
#ifdef WIN32
//disable warning: nonstandard usage of unnamed struct/union
#pragma warning(disable: 4201)
#endif
		union {
			struct {
				T _11, _12, _13, _14,
				  _21, _22, _23, _24,
				  _31, _32, _33, _34,
				  _41, _42, _43, _44;
			};
			T m[16];
		};
#ifdef WIN32
#pragma warning(default: 4201)
#endif
	public:
		gbMatrix4()
			:_11(1), _12(0), _13(0), _14(0),
			 _21(0), _22(1), _23(0), _24(0),
			 _31(0), _32(0), _33(1), _34(0),
			 _41(0), _42(0), _43(0), _44(1)
		{ ; }
		gbMatrix4(T const* p)
			:_11( p[0]), _12( p[1]), _13( p[2]), _14( p[3]),
			 _21( p[4]), _22( p[5]), _23( p[6]), _24( p[7]),
			 _31( p[8]), _32( p[9]), _33(p[10]), _34(p[11]),
			 _41(p[12]), _42(p[13]), _43(p[14]), _44(p[15])
		{ ; }
		gbMatrix4(T const& f11, T const& f12, T const& f13, T const& f14, 
				  T const& f21, T const& f22, T const& f23, T const& f24, 
				  T const& f31, T const& f32, T const& f33, T const& f34, 
				  T const& f41, T const& f42, T const& f43, T const& f44)
			:_11(f11), _12(f12), _13(f13), _14(f14),
			 _21(f21), _22(f22), _23(f23), _24(f24),
			 _31(f31), _32(f32), _33(f33), _34(f34),
			 _41(f41), _42(f42), _43(f43), _44(f44)
			{ ; }
		gbMatrix4(gbVector4<T> const& r1, gbVector4<T> const& r2, 
				  gbVector4<T> const& r3, gbVector4<T> const& r4)
			:_11(r1.GetX()), _12(r1.GetY()), _13(r1.GetZ()), _14(r1.GetW()),
			 _21(r1.GetX()), _22(r2.GetY()), _23(r2.GetZ()), _24(r2.GetW()),
			 _31(r1.GetX()), _32(r3.GetY()), _33(r3.GetZ()), _34(r3.GetW()),
			 _41(r1.GetX()), _42(r4.GetY()), _43(r4.GetZ()), _44(r4.GetW())
		{ ; }
		gbMatrix4(gbMatrix4 const& rhs)
			:_11(rhs._11), _12(rhs._12), _13(rhs._13), _14(rhs._14),
			 _21(rhs._21), _22(rhs._22), _23(rhs._23), _24(rhs._24),
			 _31(rhs._31), _32(rhs._32), _33(rhs._33), _34(rhs._34),
			 _41(rhs._41), _42(rhs._42), _43(rhs._43), _44(rhs._44)
		{ ; }
		template<typename T2>
		gbMatrix4& operator =(gbMatrix4<T2> const& rhs) {
			if( static_cast<void const*>(this) != static_cast<void const*>(&rhs) ) {
				_11 = static_cast<T>( rhs._11);  _12 = static_cast<T>( rhs._12);
				_13 = static_cast<T>( rhs._13);  _14 = static_cast<T>( rhs._14);
				_21 = static_cast<T>( rhs._21);  _22 = static_cast<T>( rhs._22);
				_23 = static_cast<T>( rhs._23);  _24 = static_cast<T>( rhs._24);
				_31 = static_cast<T>( rhs._31);  _32 = static_cast<T>( rhs._32);
				_33 = static_cast<T>( rhs._33);  _34 = static_cast<T>( rhs._34);
				_41 = static_cast<T>( rhs._41);  _42 = static_cast<T>( rhs._42);
				_43 = static_cast<T>( rhs._43);  _44 = static_cast<T>( rhs._44);
			}
			return *this;
		}
		void Identity() {
			_11 = 1;  _12 = 0;  _13 = 0;  _14 = 0;
			_21 = 0;  _22 = 1;  _23 = 0;  _24 = 0;
			_31 = 0;  _32 = 0;  _33 = 1;  _34 = 0;
			_41 = 0;  _42 = 0;  _43 = 0;  _44 = 1;
		}
		void Null() {
			_11 = 0;  _12 = 0;  _13 = 0;  _14 = 0;
			_21 = 0;  _22 = 0;  _23 = 0;  _24 = 0;
			_31 = 0;  _32 = 0;  _33 = 0;  _34 = 0;
			_41 = 0;  _42 = 0;  _43 = 0;  _44 = 0;
		}
		void SetRow(gbVector4<T> const& v, int i) { 
			int i1 = i*4;
			m[0+i1]=v.GetX();  m[1+i1]=v.GetY();  m[2+i1]=v.GetZ();  m[3+i1]=v.GetW();
		}
		void SetCol(gbVector4<T> const& v, int i) {
			m[i]=v.GetX();  m[i+4]=v.GetY();  m[i+8]=v.GetZ();  m[i+12]=v.GetW();
		}
		void SetRow(gbVector3<T> const& v, int i) { 
			int i1 = i*4;
			m[0+i1]=v.GetX();  m[1+i1]=v.GetY();  m[2+i1]=v.GetZ();  m[3+i1]=0;
		}
		void SetCol(gbVector3<T> const& v, int i) {
			m[i]=v.GetX();  m[i+4]=v.GetY();  m[i+8]=v.GetZ();  m[i+12]=0;
		}
		gbMatrix4& SwapRows(int row1, int row2) { 
			int i1 = row1*4;
			int i2 = row2*4;
			T f1  = m[i1];  T f2    = m[i1+1];  T f3    = m[i1+2];  T f4    = m[i1+3];
			m[i1] = m[i2];  m[i1+1] = m[i2+1];  m[i1+2] = m[i2+2];  m[i1+3] = m[i2+3];
			m[i2] = f1;     m[i2+1] = f2;       m[i2+2] = f3;       m[i2+3] = f4; 
			return *this;
		}
		gbMatrix4& SwapCols(int col1, int col2) { 
			int i1 = col1;
			int i2 = col2;
			T f1  = m[i1];  T f2    = m[i1+4];  T f3    = m[i1+8];  T f4     = m[i1+12];
			m[i1] = m[i2];  m[i1+4] = m[i2+4];  m[i1+8] = m[i2+8];  m[i1+12] = m[i2+12];
			m[i2] = f1;     m[i2+4] = f2;       m[i2+8] = f3;       m[i2+12] = f4; 
			return *this;
		}
		gbMatrix4& MultiplyRow(int i, T const& f) {
			int i1 = i*4; 
			m[i1] = m[i1]*f;  m[i1+1] = m[i1+1]*f;  m[i1+2] = m[i1+2]*f;  m[i1+3] = m[i1+3]*f;
			return *this;
		}
		gbMatrix4& MultiplyCol(int i, T const& f) {
			m[i] = m[i]*f;  m[i+4] = m[i+4]*f;  m[i+8] = m[i+8]*f;  m[i+12] = m[i+12]*f; 
			return *this;
		}
		gbMatrix4& AddRows(int dest, int src) {
			int i1 = dest*4;
			int i2 = src*4;
			m[i1]   = m[i1]   + m[i2];    m[i1+1] = m[i1+1] + m[i2+1];
			m[i1+2] = m[i1+2] + m[i2+2];  m[i1+3] = m[i1+3] + m[i2+3];
			return *this;
		}
		gbMatrix4& SubRows(int dest, int src) { 
			int i1 = dest*4;
			int i2 = src*4;
			m[i1]   = m[i1]   - m[i2];    m[i1+1] = m[i1+1] - m[i2+1];
			m[i1+2] = m[i1+2] - m[i2+2];  m[i1+3] = m[i1+3] - m[i2+3];
			return *this;
		}
		gbMatrix4& AddCols(int dest, int src) { 
			int i1 = dest;
			int i2 = src; 
			m[i1]   = m[i1]   + m[i2];    m[i1+4]  = m[i1+4]  + m[i2+4];
			m[i1+8] = m[i1+8] + m[i2+8];  m[i1+12] = m[i1+12] + m[i2+12]; 
			return *this;
		}
		gbMatrix4& SubCols(int dest, int src) {
			int i1 = dest;
			int i2 = src; 
			m[i1]   = m[i1]   - m[i2];    m[i1+4]  = m[i1+4]  - m[i2+4];
			m[i1+8] = m[i1+8] - m[i2+8];  m[i1+12] = m[i1+12] - m[i2+12]; 
			return *this;
		}
		gbVector4<T> GetCol(int i) const {
			return gbVector4<T>(m[i], m[i+4], m[i+8], m[i+12]);
		}
		gbVector4<T> GetRow(int i) const { 
			return gbVector4<T>(m[i*4], m[i*4+1], m[i*4+2], m[i*4+2]);
		}
		T Get(int i) const {
			return m[i];
		}
		T const* GetV() const {
			return m;
		}
		bool operator ==(gbMatrix4 const& rhs) const {
			if( this == (&rhs) ) { return true; }
			return ( (_11 == rhs._11) && (_12 == rhs._12) && (_13 == rhs._13) && (_14 == rhs._14) &&
					 (_21 == rhs._21) && (_22 == rhs._22) && (_23 == rhs._23) && (_24 == rhs._24) &&
					 (_31 == rhs._31) && (_32 == rhs._32) && (_33 == rhs._33) && (_34 == rhs._34) &&
					 (_41 == rhs._41) && (_42 == rhs._42) && (_43 == rhs._43) && (_44 == rhs._44) );
		}
		bool operator !=(gbMatrix4 const& rhs) const {
			return(!(this->operator==(rhs)));
		}
		gbMatrix4 operator +(gbMatrix4 const& rhs) const {
			return gbMatrix4( (_11+rhs._11), (_12+rhs._12), (_13+rhs._13), (_14+rhs._14),
							  (_21+rhs._21), (_22+rhs._22), (_23+rhs._23), (_24+rhs._24),
							  (_31+rhs._31), (_32+rhs._32), (_33+rhs._33), (_34+rhs._34),
							  (_41+rhs._41), (_42+rhs._42), (_43+rhs._43), (_44+rhs._44) );
		}
		gbMatrix4 operator -(gbMatrix4 const& rhs) const {
			return gbMatrix4( (_11-rhs._11), (_12-rhs._12), (_13-rhs._13), (_14-rhs._14),
							  (_21-rhs._21), (_22-rhs._22), (_23-rhs._23), (_24-rhs._24),
							  (_31-rhs._31), (_32-rhs._32), (_33-rhs._33), (_34-rhs._34),
							  (_41-rhs._41), (_42-rhs._42), (_43-rhs._43), (_44-rhs._44) );
		}
		gbMatrix4 operator *(T const& f) const {
			return gbMatrix4( (_11*f), (_12*f), (_13*f), (_14*f),
							  (_21*f), (_22*f), (_23*f), (_24*f),
							  (_31*f), (_32*f), (_33*f), (_34*f),
							  (_41*f), (_42*f), (_43*f), (_44*f) );
		}
		gbMatrix4 operator /(T const& f) const {
			return gbMatrix4( (_11/f), (_12/f), (_13/f), (_14/f),
							  (_21/f), (_22/f), (_23/f), (_24/f),
							  (_31/f), (_32/f), (_33/f), (_34/f),
							  (_41/f), (_42/f), (_43/f), (_44/f) );
		}
		gbMatrix4 operator*(gbMatrix4 const& rhs) const {
			return gbMatrix4(
					(_11*rhs._11 + _12*rhs._21 + _13*rhs._31 + _14*rhs._41),
					(_11*rhs._12 + _12*rhs._22 + _13*rhs._32 + _14*rhs._42),
					(_11*rhs._13 + _12*rhs._23 + _13*rhs._33 + _14*rhs._43),
					(_11*rhs._14 + _12*rhs._24 + _13*rhs._34 + _14*rhs._44),
					
					(_21*rhs._11 + _22*rhs._21 + _23*rhs._31 + _24*rhs._41),
					(_21*rhs._12 + _22*rhs._22 + _23*rhs._32 + _24*rhs._42),
					(_21*rhs._13 + _22*rhs._23 + _23*rhs._33 + _24*rhs._43),
					(_21*rhs._14 + _22*rhs._24 + _23*rhs._34 + _24*rhs._44),

					(_31*rhs._11 + _32*rhs._21 + _33*rhs._31 + _34*rhs._41),
					(_31*rhs._12 + _32*rhs._22 + _33*rhs._32 + _34*rhs._42),
					(_31*rhs._13 + _32*rhs._23 + _33*rhs._33 + _34*rhs._43),
					(_31*rhs._14 + _32*rhs._24 + _33*rhs._34 + _34*rhs._44),

					(_41*rhs._11 + _42*rhs._21 + _43*rhs._31 + _44*rhs._41),
					(_41*rhs._12 + _42*rhs._22 + _43*rhs._32 + _44*rhs._42),
					(_41*rhs._13 + _42*rhs._23 + _43*rhs._33 + _44*rhs._43),
					(_41*rhs._14 + _42*rhs._24 + _43*rhs._34 + _44*rhs._44)
				);
		}
		gbMatrix4& operator +=(gbMatrix4 const& rhs) {
			_11=_11+rhs._11;  _12=_12+rhs._12;  _13=_13+rhs._13;  _14=_14+rhs._14;
			_21=_21+rhs._21;  _22=_22+rhs._22;  _23=_23+rhs._23;  _24=_24+rhs._24;
			_31=_31+rhs._31;  _32=_32+rhs._32;  _33=_33+rhs._33;  _34=_34+rhs._34;
			_41=_41+rhs._41;  _42=_42+rhs._42;  _43=_43+rhs._43;  _44=_44+rhs._44;
			return *this;
		}
		gbMatrix4& operator -=(gbMatrix4 const& rhs) {
			_11=_11-rhs._11;  _12=_12-rhs._12;  _13=_13-rhs._13;  _14=_14-rhs._14;
			_21=_21-rhs._21;  _22=_22-rhs._22;  _23=_23-rhs._23;  _24=_24-rhs._24;
			_31=_31-rhs._31;  _32=_32-rhs._32;  _33=_33-rhs._33;  _34=_34-rhs._34;
			_41=_41-rhs._41;  _42=_42-rhs._42;  _43=_43-rhs._43;  _44=_44-rhs._44;
			return *this;
		}
		gbMatrix4& operator *=(T const& f) {
			_11=_11*f;  _12=_12*f;  _13=_13*f;  _14=_14*f;
			_21=_21*f;  _22=_22*f;  _23=_23*f;  _24=_24*f;
			_31=_31*f;  _32=_32*f;  _33=_33*f;  _34=_34*f;
			_41=_41*f;  _42=_42*f;  _43=_43*f;  _44=_44*f;
			return *this;
		}
		gbMatrix4& operator /=(T const& f) {
			_11=_11/f;  _12=_12/f;  _13=_13/f;  _14=_14/f;
			_21=_21/f;  _22=_22/f;  _23=_23/f;  _24=_24/f;
			_31=_31/f;  _32=_32/f;  _33=_33/f;  _34=_34/f;
			_41=_41/f;  _42=_42/f;  _43=_43/f;  _44=_44/f;
			return *this;
		}
		gbMatrix4& operator *=(gbMatrix4 const& rhs) {
			gbMatrix4<T> tmp( (*this) * rhs );
			(*this) = tmp;
			return *this;
		}
		gbVector4<T> MultiplyVector(gbVector4<T> const& v) const {
			return gbVector4<T>(( v.GetX()*_11 + v.GetY()*_12 + v.GetZ()*_13 + v.GetW()*_14 ),
								( v.GetX()*_21 + v.GetY()*_22 + v.GetZ()*_23 + v.GetW()*_24 ),
								( v.GetX()*_31 + v.GetY()*_32 + v.GetZ()*_33 + v.GetW()*_34 ),
								( v.GetX()*_41 + v.GetY()*_42 + v.GetZ()*_43 + v.GetW()*_44 ) );
		}
		gbMatrix4& Transpose() {
			T f;
			f = _21; _21 = _12; _12 = f;
			f = _31; _31 = _13; _13 = f;
			f = _41; _41 = _14; _14 = f;
			f = _32; _32 = _23; _23 = f;
			f = _42; _42 = _24; _24 = f;
			f = _43; _43 = _34; _34 = f;
			return *this;
		}
		T GetDeterminant() const {
			return T(_11 * (_22*_33*_44 + _23*_34*_42 + _24*_32*_43 - _42*_33*_24 - _43*_34*_22 - _44*_32*_23) -
					 _12 * (_21*_33*_44 + _23*_34*_41 + _24*_31*_43 - _41*_33*_24 - _43*_34*_21 - _44*_31*_23) +
					 _13 * (_21*_32*_44 + _22*_34*_41 + _24*_31*_42 - _41*_32*_24 - _42*_34*_21 - _44*_31*_22) -
					 _14 * (_21*_32*_43 + _22*_33*_41 + _23*_31*_42 - _41*_32*_23 - _42*_33*_21 - _43*_31*_22));
		}
		T GetInnerDeterminant3() const {
			return T( _11 * _22 * _33 + _12 * _23 * _31 + _13 * _21 * _32 - 
					  _31 * _22 * _13 - _32 * _23 * _11 - _33 * _21 * _12 );
		}
		T GetInnerDeterminant2() const {
			return T(_11 * _22 - _21 * _12);
		}
		bool IsOrthogonal() const {
			gbMatrix4 t(this);
			T f = t.GetDeterminant();
			t.Transpose();
			t *= (*this);
			if((t == gbMatrix4()) && (f == 1)) {
				return true;
			} else {
				return false;
			}
		}
	};
	template<typename T>
	::std::ostream& operator<<(::std::ostream& os, gbMatrix4<T> const& m) {
		os << std::endl << "gbMatrix4 - ";
		for(int i=0; i<16; i++) { os << m.Get(i) << " "; if((i+1)%4 == 0) { os << std::endl << "            "; } }
		return os;
	}

	template<typename T>
	class gbComplex {
	public:
#ifdef WIN32
//disable warning: nonstandard usage of unnamed struct/union
#pragma warning(disable: 4201)
#endif
	  union {
		  struct {
			  T _r, _i;
		  };
		  T c[2];
	  };
#ifdef WIN32
#pragma warning(default: 4201)
#endif
	public:
	  gbComplex(T const& real, T const& imag) 
		  :_r(real), _i(imag)
	  { ; }
	  gbComplex(T const* p)
		  :_r(p[0]), _i(p[1])
	  { ; }
	  gbComplex(gbComplex const& rhs)
		  :_r(rhs.r), _i(rhs.i)
	  { ; }
	  template<typename T_2>
	  gbComplex& operator=(gbComplex<T_2> const& rhs) {
		  if( static_cast<void const*>(this) != static_cast<void const*>(&rhs) ) {
			  _r = static_cast<T>(rhs._r);
			  _i = static_cast<T>(rhs._i);
		  }
		  return *this;
	  }
	  void SetReal(T const& real) {
		  _r = real;
	  }
	  void SetImag(T const& imag) {
		  _i = imag;
	  }
	  void SetComp(T const& real, T const& imag) {
		  _r = real;
		  _i = imag;
	  }
	  void SetComp(T const* p) {
		  _r = p[0];
		  _i = p[1];
	  }
	  T GetReal() const {
		  return _r;
	  }
	  T GetImag() const {
		  return _i;
	  }
	  T const* GetV() const {
		  return c;
	  }
	  T GetLen() const {
		  return static_cast<T>( sqrt( static_cast<double>(_r*_r + _i*_i) ) );
	  }
	  T GetAngle() const {
		  if(_i < 0) {
			  return static_cast<T>( acos(-1.0 * static_cast<double>(_r) / 
										  sqrt( static_cast<double>(_r*_r+_i*_i) ) ) - M_PI );
		  } else {
			  return static_cast<T>( acos( static_cast<double>(_r) / 
										   sqrt( static_cast<double>(_r*_r+_i*_i) ) ) );
		  }
	  }
	  gbComplex& Conjugate() {
		  _i = _i * -1;
		  return *this;
	  }
	  bool operator==(gbComplex const& rhs) const {
		  return( (_r == rhs._r) && (_i == rhs._i) );
	  }
	  bool operator!=(gbComplex const& rhs) const {
		  return(!(this->operator==(rhs)));
	  }
	  gbComplex operator +(gbComplex const& rhs) const {
		  return gbComplex(_r+rhs._r, _i+rhs._i);
	  }
	  gbComplex operator -(gbComplex const& rhs) const {
		  return gbComplex(_r-rhs._r, _i-rhs._i);
	  }
	  gbComplex operator *(gbComplex const& rhs) const {
		  return gbComplex(_r*rhs._r - _i*rhs._i, _r*rhs._i + _i*rhs._r);
	  }
	  gbComplex operator /(gbComplex const& rhs) const {
		  return gbComplex( (_r*rhs._r+_i*rhs._i)/(rhs._r*rhs._r+rhs._i*rhs._i), 
							(_i*rhs._r-_r*rhs._i)/(rhs._r*rhs._r+rhs._i*rhs._i) );
	  }
	  gbComplex& operator+=(gbComplex const& rhs) {
		  _r = _r + rhs._r;
		  _i = _i + rhs._i;
		  return *this;
	  }
	  gbComplex& operator-=(gbComplex const& rhs) {
		  _r = _r - rhs._r;
		  _i = _i - rhs._i;
		  return *this;
	  }
	  gbComplex& operator*=(gbComplex const& rhs) {
		  T t_r = _r;
		  T t_i = _i;
		  _r = t_r*rhs._r - t_i*rhs._i;
		  _i = t_r*rhs._i + t_i*rhs._r;
		  return *this;
	  }
	  gbComplex& operator/=(gbComplex const& rhs) {
		  T t_r = _r;
		  T t_i = _i;
		  _r = (t_r*rhs._r+t_i*rhs._i)/(rhs._r*rhs._r+rhs._i*rhs._i);
		  _i = (t_i*rhs._r-t_r*rhs._i)/(rhs._r*rhs._r+rhs._i*rhs._i);
		  return *this;
	  }
	};
	template<typename T>
	::std::ostream& operator<<(::std::ostream& os, gbComplex<T> const& c) {
		return os << "gbComplex - Real:" << c.GetReal() << " Imag:" << c.GetImag();
	}
}

#endif
