/**
 * @file include/gbSprite.hpp
 *
 * ghulbusGraphics - The Ghulbus Graphics Library
 *
 * @brief gbGraphics Sprite header file
 *
 * @version 0.1
 * @author Andreas Weis
 *
 */

#ifndef _GHULBUSGRAPHICS_SPRITE_HPP_INCLUDE_GUARD_
#define _GHULBUSGRAPHICS_SPRITE_HPP_INCLUDE_GUARD_

#include "gbException.hpp"
#include "gbColor.hpp"
#include "gbTexture.hpp"
#include "gbGraphicsObject.hpp"
#include "gbMath.hpp"

#ifdef _GHULBUSGRAPHICS__USE_SDL_
#	ifdef WIN32
#		include <SDL.h>
#		define WIN32_LEAN_AND_MEAN
#		include <windows.h>
#	else
#		include <SDL/SDL.h>
#	endif
#	include <GL/glew.h>
#	include <GL/gl.h>
#	include <GL/glu.h>
#endif

#ifdef _GHULBUSGRAPHICS__USE_D3D9_
#	ifndef WIN32
#		undef _GHULBUSGRAPHICS__USE_D3D9_
#	else
#		define WIN32_LEAN_AND_MEAN
#		include <windows.h>
#		include <d3d9.h>
#		include <d3dx9.h>
#	endif
#endif

namespace GhulbusGraphics {
	/** Sprite
	 */
	class gbSprite: public gbGraphicsObject {
	protected:
		gbTexture* m_texture;						///< sprite texture
		GhulbusMath::gbVector3<float> m_position;	///< sprite position in world space
	protected:
		/** Destructor
		 */
		virtual ~gbSprite();
	public:
		/** Constructor
		 * @note Use a manager to create a Sprite
		 */
		gbSprite();
		/** Change the sprites position in the world
		 * @param[in] new_pos New position of the sprite in world space
		 */
		virtual void SetPosition(GhulbusMath::gbVector3<float> const& new_pos);
		/** Get the current position of the sprite
		 * @param[out] current_pos Current position of the sprite in world space
		 */
		virtual void GetPosition(GhulbusMath::gbVector3<float>* current_pos);
		/** Get the appropriate texture of the sprite
		 * @return The sprite's texture
		 * @note Use the acquired texture to change the image displayed by the sprite
		 */
		virtual gbTexture* GetTexture();
	};

#ifdef _GHULBUSGRAPHICS__USE_SDL_
	class gbSprite_SDL: public gbSprite {
	private:
		GhulbusMath::gbVector3<float> m_spriteFaceLL;
		GhulbusMath::gbVector3<float> m_spriteFaceLR;
		GhulbusMath::gbVector3<float> m_spriteFaceUL;
		GhulbusMath::gbVector3<float> m_spriteFaceUR;
		int m_scrWidth;
		int m_scrHeight;
		GhulbusMath::gbVector2<float> m_texCoords;
	protected:
		virtual ~gbSprite_SDL();
	public:
		gbSprite_SDL(gbTexture_SDL* texture, int scr_width, int scr_height);
		virtual void SetPosition(GhulbusMath::gbVector3<float> const& new_pos);
		virtual void Render();
		virtual void Release();
		virtual void SetTexCoords();
	};
#endif

#ifdef _GHULBUSGRAPHICS__USE_D3D9_
	class gbSprite_D3D9: public gbSprite {
	private:
		LPD3DXSPRITE m_lpSprite;
	protected:
		virtual ~gbSprite_D3D9();
	public:
		gbSprite_D3D9(LPD3DXSPRITE sprite, gbTexture_D3D9* texture);
		virtual void Render();
		virtual void Release();
	};
#endif
	/*-------------------------------------
	 | Extensive Documentation:           |
	 *------------------------------------*/
	/** 
	 * @class gbSprite
	 *
	 * @details A sprite is a 2 dimensional object that can be used to display
	 * images in a 3d scene. In contrast of simple blitting to the framebuffer,
	 * a sprite is actual textured geometry positioned in 3d space.
	 * Use the gbSprite class to integrate static image data in a scene.
	 * In contrast to a gbTexture, which is nothing but an abstract wrapper for
	 * data, the gbSprite is actual geometry residing in your scene.
	 *
	 * @section gbsprite_d3d9 Implementation Details Direct3D9
	 * The D3D implementation utilizes the D3DXSPRITE class provided by the
	 * D3D extension library. Since the D3DXSPRITE is already quite easy to
	 * use, most methods are mere wrappers for the respective D3DX functions.
	 * 
	 * @section gbsprite_sdl Implementation Details SDL
	 * The OpenGL implementation manages the underlying geometry all by itself.
	 * Since this approach inevitably requires some math, the coordinates
	 * of the D3DXSPRITE were used as a reference.
	 * The underlying geometry consists of two triangles per sprite, forming
	 * a rectangular surface.
	 * Note that in OpenGL the actual texture size may be larger than requested
	 * due to GLEW_ARB_texture_non_power_of_two (see @ref gbtexture_sdl "this section"
	 * for details), which requires manual adjustment of texture coordinates in
	 * SetTexCoords().
	 *
	 */
}

#endif
