/**
 * @file include/gbTurtle.hpp
 *
 * ghulbusGraphics - The Ghulbus Graphics Library
 *
 * @brief gbGraphics Turtle Plotter
 *
 * @version 0.1
 * @author Andreas Weis
 *
 */

#ifndef _GHULBUSGRAPHICS_TURTLE_HPP_INCLUDE_GUARD_
#define _GHULBUSGRAPHICS_TURTLE_HPP_INCLUDE_GUARD_

#include "gbException.hpp"
#include "gbColor.hpp"
#include "gb2D.hpp"
#include <cmath>

namespace GhulbusGraphics {
	/**
	 * @brief Turtle Graphics
	 * 
	 * A simple plotter for Logo-style turtle graphics
	 */
	class gbTurtle {
	private:
		gb2D*   m_gb2D;		///< gb2DManager structure used for painting
		double  m_posX;		///< X position in turtle coordinates
		double  m_posY;		///< Y position in turtle coordinates
		GBCOLOR m_penColor;	///< current pen-color
		double  m_angle;	///< angle relative to positive turtle-coordinate-x-axis
		bool    m_penDown;	///< if the pen is up, the turtle won't paint while moving
		double	m_height;
		double  m_width;
	public:
		/** Constructor.
		 * @param lpgb2D A pointer to a @ref gb2D object
		 * @throw Ghulbus::gbException GB_ILLEGALPARAMETER
		 */
		gbTurtle(gb2D* lpgb2D);
		/** Destructor
		 */
		~gbTurtle();

		/** @name Moving the turtle:
		 * @{
		 */
		/** Move the turtle.
		 *
		 * Moves the turtle in the direction it is currently facing.
		 * @param units Number of pixel the turtle is moving forward.
		 *              If negative, the turtle moves backwards.
		 */
		void Go(int units);
		/** Turn the turtle to the left.
		 *
		 * @param iAngle Angle that the turtle shall turn in degrees
		 */
		void TurnLeft(int iAngle);
		/** Turn the turtle to the right.
		 *
		 * @param iAngle Angle that the turtle shall turn in degrees
		 */
		void TurnRight(int iAngle);
		/** Turn the turtle at a specific angle
		 *
		 * @param iAngle Angle that the turtle shall be facing in degrees
		 */
		void SetAngle(int iAngle);
		/** Turn the turtle to the left.
		 *
		 * @param fAngle Angle that the turtle shall turn in radians
		 */
		void TurnLeftRad(double fAngle);
		/** Turn the turtle to the right.
		 *
		 * @param fAngle Angle that the turtle shall turn in radians
		 */
		void TurnRightRad(double fAngle);
		/** Turn the turtle at a specific angle
		 *
		 * @param fAngle Angle that the turtle shall be facing in radians
		 */
		void SetAngleRad(double fAngle);
		/** Set the turtle to a specific position
		 *
		 * @param X New X coordinate
		 * @param Y New Y coordinate
		 */
		void Goto(int X, int Y);
		/** Reset the turtle to its initial position, angle and color.
		 * @attention No drawing occurs when resetting the turtle!
		 */
		void Reset();
		//////////////////////////////////////////////////////////////////// @}
		

		/** @name Manipulating the pen:
		 * @{
		 */
		/** Set the pen's color.
		 *
		 * @param r Specifying the new color's red component (0-255)
		 * @param g Specifying the new color's green component (0-255)
		 * @param b Specifying the new color's blue component (0-255)
		 */
		void SetColor(GBCOLOR_COMPONENT r, GBCOLOR_COMPONENT g, GBCOLOR_COMPONENT b);
		/** Move the pen up.
		 */
		void PenUp();
		/** Move the pen down.
		 */
		void PenDown();
		/** Toggle the pen.
		 */
		void PenToggle();
		//////////////////////////////////////////////////////////////////// @}


		/** @name Functions for status-checking:
		 * @{
		 */
		/** Check the status of the pen.
		 *
		 * @return true if the pen is down, false if not.
		 */
		bool IsPenDown();
		/** Draw a representation of the turtle showing its position and angle.
		 */
		void Show();
		/** Get the position of the turtle in X.
		 * 
		 * @return The current X coordinate of the turtle's position.
		 */
		int GetPosX();
		/** Get the position of the turtle in Y.
		 * 
		 * @return The current Y coordinate of the turtle's position.
		 */
		int GetPosY();
		/** Get the angle in which the turtle is currently facing.
		 *
		 * @return The angle in which the turtle is currently directed in degrees.
		 */
		int GetAngle();
		/** Get the angle in which the turtle is currently facing.
		 *
		 * @return The angle in which the turtle is currently directed in radians.
		 */
		double GetAngleRad();
		/// @}
	private:
		/** Convert turtle-coordinates to gb2D-screen-coordinates
		 */
		inline double XCord(double xRaw) { return ((m_width /2.0) + xRaw); }
		/** Convert turtle-coordinates to gb2D-screen-coordinates
		 */
		inline double YCord(double yRaw) { return ((m_height/2.0) - yRaw); }
		//CONSTANTS:
		static const double GB2_PI;
		static const double GBPI_180;
		static const double GB_EPSILON;
	};
	/*-------------------------------------
	 | Extensive Documentation:           |
	 *------------------------------------*/
	/** 
	 * @class gbTurtle
	 *
	 * @details The class is drawing directly to the gb2D surface
	 *  and keeps no history of its former movements. 
	 *  That means, once you clear the surface everything the turtle 
	 *  drew up to that point will be lost.
	 *
	 *  The turtle starts in the middle of the screen heading to 
	 *  the right. Whenever the turtle is moved with the pen down, 
	 *  it draws a line in the color of the pen, marking its route.
	 *
	 *  The turtle's coordinate system has its origin in the middle
	 *  of the gb2D surface with its X-axis pointing right and its
	 *  Y-axis pointing down.
	 *
	 */
}

#endif
